@@ -1,3 +1,18 @@
+# Licensed to Tomaz Muraus under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# Tomaz muraus licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import os
 import logging
 
@@ -28,6 +43,8 @@ def run():
                       help='API username')
     parser.add_option('--key', dest='api_key',
                       help='API key')
+    parser.add_option('--restore', dest='restore', action="store_true",
+                      help='Restore from')
     parser.add_option('--container-name', dest='container_name',
                       default='file_syncer',
                       help='Name of the container storing the files')
@@ -82,4 +99,7 @@ def run():
                         exclude_patterns=exclude_patterns,
                         logger=logger,
                         concurrency=int(options.concurrency))
-    syncer.sync()
+    if options.restore:
+      syncer.restore()
+    else:
+      syncer.sync()
